
/* STRINLIB.C 1.6 */

#include "strinlib.h"

static char convert_ascii[] = "<>&";
static char convert_html[][9] = { "&lt;", "&gt;", "&amp;", "&agrave;", "&egrave;", "&igrave;", "&ograve;", "&ugrave;", "&eacute;" };
static char convert_accent[][3] = { "a'", "e'", "i'", "o'", "u'" };
static char ascii_accent[] = "";


int is_spacer(char ch)

   {return ( ((ch == ' ') || (ch == '\t')) ? 1 : 0 );}


int isnot_spacer(char ch)

   {return ( ((ch == ' ') || (ch == '\t')) ? 0 : 1 );}


int is_crlf(char ch)

   {return ( ((ch == '\012') || (ch == '\015')) ? 1 : 0 );}


int isnot_crlf(char ch)

   {return ( ((ch == '\012') || (ch == '\015')) ? 0 : 1 );}


int is_cursor(char ch)

   {return ( ((ch == '\012') || (ch == '\015') || (ch == ' ') || (ch == '\t')) ? 1 : 0 );}


int isnot_cursor(char ch)

   {return ( ((ch == '\012') || (ch == '\015') || (ch == ' ') || (ch == '\t')) ? 0 : 1 );}


char basechar(int n, int radix)

   {if ((n < 0) || (n >= radix))
        return '#';
    else if (n < 10)
        return ('0' + n);
    else
        return ('A' + n - 10);
   }


int strlcmp(char *s1, char *s2, int nmean, int sensit)

   {if (sensit)
       {if (nmean == -1)
            return strcmp(s1, s2);
        else
            return strncmp(s1, s2, nmean);
       }
    else
       {if (nmean == -1)
            return stricmp(s1, s2);
        else
            return strnicmp(s1, s2, nmean);
       }
   }


int in_wordset(char *s, strset_t wset)

   {int i = 0;
    int rv = 0;
    while ((wset[i][0] != '\0') && (!rv))
       {rv = !strcmp(s, wset[i]);
        i++;}
    return rv;
   } 


char *revstr(char *s)

   {int i, j, l;
    char ch;
    l = strlen(s);
    for(i = 0, j = l-1; i < l/2; i++, j--)
       {ch = s[i];
        s[i] = s[j];
        s[j] = ch;}
    return s;
   }  


char *mvleft(char *s, int startpos, int delta)

   {int i;
    if ((startpos < 0) || (startpos > strlen(s)))
        return NULL;
    else if (delta == 0)
        return s;
    else if (s != NULL)
       {i = startpos - delta;
        if (i < 0)
            return NULL;
        else
           {for (; s[startpos] != '\0'; startpos++)
                s[i++] = s[startpos];
            s[i] = '\0';
	    return s;}
       }
    else
        return NULL;
   }


char *mvright(char *s, int startpos, int delta, int slen)

   {int i, j;
    if ((startpos < 0) || (startpos > strlen(s)))
        return NULL;
    else if (delta == 0)
        return s;
    else if (s != NULL)
       {i = strlen(s) + delta;
        if (i > slen)
            i = slen;
        s[i--] = '\0';
        j = i - delta;
        for (; i >= startpos + delta; i--)
            s[i] = s[j--];
        return s;
       }
    else
        return NULL;
   }


char *rmlchar(char *s, IS_CHAR_F f)

   {int i = 0;
    while (f(s[i]))
        i++;
    if (i)
        mvleft(s, i, i);
    return s;
   }


char *rmrchar(char *s, IS_CHAR_F f)

   {int i;
    i = strlen(s);
    while (i && (f(s[i-1])))
	s[--i]='\0';
    return s;
   }


char *rmlspc(char *s)

   {int i = 0;
    while (is_spacer(s[i]))
        i++;
    if (i)
        mvleft(s, i, i);
    return s;
   }


char *rmrcursor(char *s)

   {int i;
    i = strlen(s);
    while (i && (is_cursor(s[i-1])))
	s[--i]='\0';
    return s;
   }


char *rmrcrlf(char *s)

   {int i;
    i = strlen(s);
    while (i && (is_crlf(s[i-1])))
	s[--i]='\0';
    return s;
   }


char *strsqz(char *s)

   {int i = 0;
    int j;
    rmlspc(s);
    while (s[i] != '\0')
       {if (is_cursor(s[i]))
	   {j = (++i);
	    while (is_cursor(s[j]))
	       j++;
            if (j > i)
               mvleft(s, j, j-i);
	    s[i-1] = ' ';}
        i++;
       }
    rmrcursor(s);
    return s;
   }


char *strupfirst(char *s)

   {int i;
    int upnext = 1;
    for (i = 0; i < strlen(s); i++)
       {s[i] = (upnext ? toupper(s[i]) : tolower(s[i]));
        upnext = ((is_cursor(s[i])) ? 1 : 0);}
    return s;
   }


char *strcpyto(char endch, char *s, char *ds, int dslen)

   {int i = 0;
    while ((s[i] != endch) && (s[i] != '\0') && (i < dslen))
       {ds[i] = s[i];
        i++;}
    ds[i] = '\0';
    s += i;
    if (*s == '\0')
        s = NULL;
    return s;
   }


char *strcpywhile(IS_CHAR_F f, char *s, char *ds, int dslen)

   {int i = 0;
    while (f(s[i]) && (s[i] != '\0') && (i < dslen))
       {ds[i] = s[i];
        i++;}
    ds[i] = '\0';
    s += i;
    if (*s == '\0')
        s = NULL;
    return s;
   }


char *strskipto(char endch, char *s)

   {while ((*s != endch) && (*s != '\0'))
        s++;
    if (*s == '\0')
        s = NULL;
    return s;
   }


char *strskipwhile(IS_CHAR_F f, char *s)

   {while (f(*s) && (*s != '\0'))
        s++;
    if (*s == '\0')
        return NULL;
    else
        return s;
   }


char *skipspc(char *s)

   {while (is_spacer(*s) && (*s != '\0'))
        s++;
    if (*s == '\0')
        return NULL;
    else
        return s;
   }


char *wordof(char *s, char *p, char *ds, int dslen)

   {char *q;
    int i = 0;
    q = p;
    while ((q >= s) && (!is_cursor(*q)))
        q--;
    q++;
    while ((*q != '\0') && (!is_cursor(*q)) && (i < dslen))
       {ds[i++] = *q;
        q++;}
    ds[i] = '\0';
    return ds;
   }


char *itofa(int n, char *s, int radix, int ndigits)

   {int i = 0;
    while ((n > 0) && (i < ndigits))
       {s[i] = basechar(n % radix, radix);
        n /= radix;
        i++;}
    while (i < ndigits)
        s[i++] = '0';
    s[i] = '\0';
    revstr(s);
    return s;
   }


char *atoh(char *s, int slen)

   {int i = 0;
    int j, m, i0;
    while (s[i] != '\0')
       {for (j = 0; convert_ascii[j] != '\0'; j++)
            if (s[i] == convert_ascii[j])
               {i0 = i;
                m = strlen(convert_html[j]);
	        mvright(s, i+1, m-1, slen);
                for(; i < i0+m; i++)
                    s[i] = convert_html[j][i-i0];
               }
        i++;
       }
    return s;
   }


char *htoa(char *s)

   {int j = 0;
    int m;
    char *p;
    while (convert_ascii[j] != '\0')
       {m = strlen(convert_html[j]) - 1;
        p = strstr(s, convert_html[j]);
        while (p != NULL)
           {*p = convert_ascii[j];
	    mvleft(p, 1+m, m);
            p = strstr(s, convert_html[j]);}
        j++;
       }
    return s;
   }


char *spctound(char *s)

   {int i;
    for (i = 0; i < strlen(s); i++)
         if (is_spacer(s[i]))
             s[i] = '_';
    return s;
   }


char *undtospc(char *s)

   {int i;
    for (i = 0; i < strlen(s); i++)
         if (s[i] == '_')
             s[i] = ' ';
    return s;
   }


char *coupletoaccent(char *s)

   {int j = 0;
    int m;
    char *p;
    char ds[MAX_WORDLEN+1];
    while (ascii_accent[j] != '\0')
       {m = strlen(convert_accent[j]) - 1;
        p = strstr(s, convert_accent[j]);
        while (p != NULL)
           {if (!in_wordset(wordof(s, p, ds, MAX_WORDLEN), no_convacc_set))
               {*p = ascii_accent[j];
	        mvleft(p, 1+m, m);}
            p++;
            p = strstr(p, convert_accent[j]);}
        j++;
       }
    return s;
   }


char *itmon(int n)

   {switch (n)
       {case 1: return "Gennaio";
        case 2: return "Febbraio";
        case 3: return "Marzo";
        case 4: return "Aprile";
        case 5: return "Maggio";
        case 6: return "Giugno";
        case 7: return "Luglio";
        case 8: return "Agosto";
        case 9: return "Settembre";
        case 10: return "Ottobre";
        case 11: return "Novembre";
        case 12: return "Dicembre";
	default: return "";}
   }

